@@ -3,42 +3,18 @@ use crate::quadtree::aabb::AABB;
 use crate::quadtree::error::InsertError;
 use crate::quadtree::free_list;
 use crate::quadtree::free_list::{FreeList, IndexType};
-use crate::quadtree::node::{Node, NodeElementCountType};
+use crate::quadtree::node::Node;
 use crate::quadtree::node_data::{NodeData, NodeIndexType};
 use crate::quadtree::node_info::NodeInfo;
 use crate::quadtree::node_list::NodeList;
 use crate::quadtree::quad_rect::QuadRect;
 use crate::quadtree::quadrants::Quadrants;
+pub use crate::quadtree::quadtree_element::{ElementIdType, QuadTreeElement};
 use smallvec::SmallVec;
 use std::collections::HashSet;
 
 // TODO: Add range query: Query using intersect_aabb() or intersect_generic()
 
-/// Each node must have less than the maximum allowed number of elements.
-const MAX_NUM_ELEMENTS: NodeElementCountType = 1; // TODO: Make parameter of tree
-
-/// We use this value to determine whether a node can be split.
-const SMALLEST_CELL_SIZE: i32 = 1; // TODO: Make parameter of tree
-
-/// Alias for all traits required for an element ID.
-pub trait ElementIdType: Default + std::cmp::Eq + std::hash::Hash + Copy {}
-
-/// Helper implementation to automatically derive the [`ElementIdType`] trait
-impl<T> ElementIdType for T where T: Default + std::cmp::Eq + std::hash::Hash + Copy {}
-
-/// Represents an element in the QuadTree.
-#[derive(Debug, PartialEq, Eq, Default, Copy, Clone)]
-pub struct QuadTreeElement<ElementId = u32>
-where
-    ElementId: ElementIdType,
-{
-    // TODO: Split element into two structs: One containing the ID, another one containing the coordinates only. This allows aligning the elements much better. Benchmark!
-    /// Stores the ID for the element (can be used to refer to external data).
-    pub id: ElementId,
-    /// The axis-aligned bounding box of the element.
-    pub rect: AABB,
-}
-
 /// Represents an element node in the quadtree.
 ///
 /// # Remarks
@@ -62,9 +38,12 @@ pub struct QuadTree<ElementId = u32>
 where
     ElementId: ElementIdType,
 {
-    /// Stores all the elements in the quadtree.
+    /// Stores all the IDs fo the elements in the quadtree.
     /// An element is only inserted once to the quadtree no matter how many cells it occupies.
-    elements: FreeList<QuadTreeElement<ElementId>>,
+    element_ids: FreeList<ElementId>,
+    /// Stores all the rectangles of the elements in the quadtree.
+    /// An element is only inserted once to the quadtree no matter how many cells it occupies.
+    element_rects: FreeList<AABB>,
     /// Stores all the element nodes in the quadtree.
     /// For each cell occupied by a `QuadTreeElement`, we store
     /// a `QuadTreeElementNode`.
@@ -81,15 +60,10 @@ where
     free_node: free_list::IndexType,
     /// Stores the maximum depth allowed for the quadtree.
     max_depth: u32,
-}
-
-impl<ElementId> QuadTreeElement<ElementId>
-where
-    ElementId: ElementIdType,
-{
-    pub fn new(id: ElementId, rect: AABB) -> Self {
-        Self { id, rect }
-    }
+    /// Stores the maximum number of elements allowed before a node splits.
+    max_num_elements: u32,
+    /// We use this value to determine whether a node can be split.
+    smallest_cell_size: u32,
 }
 
 impl<ElementId> QuadTree<ElementId>
@@ -97,17 +71,27 @@ where
     ElementId: ElementIdType,
 {
     pub fn default() -> Self {
-        Self::new(QuadRect::default(), 8)
+        Self::new(QuadRect::default(), 8, 16, 1)
     }
 
-    pub fn new(root_rect: QuadRect, max_depth: u32) -> Self {
+    pub fn new(
+        root_rect: QuadRect,
+        max_depth: u32,
+        max_num_elements: u32,
+        smallest_cell_size: u32,
+    ) -> Self {
+        assert!(max_num_elements > 0);
+        assert!(smallest_cell_size > 0);
         Self {
-            elements: FreeList::default(),
+            element_ids: FreeList::default(),
+            element_rects: FreeList::default(),
             element_nodes: FreeList::default(),
             nodes: vec![Node::default()],
             root_rect,
             free_node: free_list::SENTINEL,
             max_depth,
+            max_num_elements,
+            smallest_cell_size,
         }
     }
 
@@ -117,8 +101,12 @@ where
             return Err(InsertError::OutOfBounds);
         }
 
+        let max_num_elements = self.max_num_elements;
+
         // Insert the actual element.
-        let element_idx = self.elements.insert(element);
+        let element_idx = self.element_ids.insert(element.id);
+        let element_rect_idx = self.element_rects.insert(element.rect);
+        debug_assert_eq!(element_idx, element_rect_idx);
 
         let mut to_process: SmallVec<[NodeData; 128]> =
             smallvec::smallvec![self.get_root_node_data()];
@@ -138,8 +126,8 @@ where
                     (node.element_count, node.first_child_or_element)
                 };
 
-                let can_split = leaf.can_split_further(SMALLEST_CELL_SIZE, self.max_depth);
-                let node_is_full = element_count >= MAX_NUM_ELEMENTS;
+                let can_split = leaf.can_split_further(self.smallest_cell_size, self.max_depth);
+                let node_is_full = element_count >= max_num_elements;
 
                 let must_store_element = !node_is_full || !can_split;
                 if must_store_element {
@@ -179,7 +167,7 @@ where
         // For each element in the list ...
         while element_node_index != free_list::SENTINEL {
             let element_node = unsafe { *self.element_nodes.at(element_node_index) };
-            let element = unsafe { *self.elements.at(element_node.element_idx) };
+            let element = unsafe { *self.element_rects.at(element_node.element_idx) };
 
             self.assign_element_to_child_nodes(
                 mx,
@@ -229,12 +217,12 @@ where
         my: i32,
         first_child_index: free_list::IndexType,
         element_index: free_list::IndexType,
-        element: &QuadTreeElement<ElementId>,
+        element_rect: &AABB,
     ) {
-        let insert_top = element.rect.tl.y <= my;
-        let insert_bottom = element.rect.br.y > my;
-        let insert_left = element.rect.tl.x <= mx;
-        let insert_right = element.rect.br.x > mx;
+        let insert_top = element_rect.tl.y <= my;
+        let insert_bottom = element_rect.br.y > my;
+        let insert_left = element_rect.tl.x <= mx;
+        let insert_right = element_rect.br.x > mx;
 
         if insert_top & insert_left {
             self.insert_element_in_child_node(first_child_index + 0, element_index);
@@ -297,9 +285,9 @@ where
 
             while element_node_idx != free_list::SENTINEL {
                 let elem_node = *unsafe { self.element_nodes.at(element_node_idx) };
-                let elem = unsafe { self.elements.at(elem_node.element_idx) };
+                let elem_id = unsafe { self.element_ids.at(elem_node.element_idx) };
 
-                if elem.id == element.id {
+                if *elem_id == element.id {
                     debug_assert!(!element_found);
                     element_found = true;
 
@@ -352,7 +340,8 @@ where
         }
 
         if found_element_idx != free_list::SENTINEL {
-            self.elements.erase(found_element_idx);
+            self.element_ids.erase(found_element_idx);
+            self.element_rects.erase(found_element_idx);
             true
         } else {
             false
@@ -568,7 +557,8 @@ where
             }
         }
 
-        debug_assert!(count >= self.elements.debug_len());
+        debug_assert!(count >= self.element_ids.debug_len());
+        debug_assert!(count >= self.element_rects.debug_len());
         count
     }
 
@@ -608,7 +598,7 @@ where
     where
         T: IntersectsWith<AABB>,
     {
-        let capacity = leaves.len() * MAX_NUM_ELEMENTS as usize;
+        let capacity = leaves.len() * self.max_num_elements as usize;
         let mut node_set = HashSet::with_capacity(capacity);
 
         while !leaves.is_empty() {
@@ -619,12 +609,13 @@ where
             let mut elem_node_idx = leaf.first_child_or_element;
             while elem_node_idx != free_list::SENTINEL {
                 let elem_node = unsafe { self.element_nodes.at(elem_node_idx) };
-                let elem = unsafe { self.elements.at(elem_node.element_idx) };
+                let elem_rect = unsafe { self.element_rects.at(elem_node.element_idx) };
 
                 // Depending on the size of the quadrant, the candidate element
                 // might still not be covered by the search rectangle.
-                if rect.intersects_with(&elem.rect) {
-                    let _was_known = node_set.insert(elem.id);
+                if rect.intersects_with(&elem_rect) {
+                    let elem_id = *unsafe { self.element_ids.at(elem_node.element_idx) };
+                    let _was_known = node_set.insert(elem_id);
                 }
 
                 elem_node_idx = elem_node.next;
@@ -645,18 +636,24 @@ where
 #[cfg(test)]
 pub(crate) fn build_test_tree() -> QuadTree {
     let quad_rect = QuadRect::new(-20, -20, 40, 40);
-    let mut tree = QuadTree::new(quad_rect, 1);
+    let mut tree = QuadTree::new(quad_rect, 1, 1, 1);
     // top-left
-    tree.insert(QuadTreeElement::new(1000, AABB::new(-15, -15, -5, -5)));
-    tree.insert(QuadTreeElement::new(1001, AABB::new(-20, -20, -18, -18)));
+    tree.insert(QuadTreeElement::new(1000, AABB::new(-15, -15, -5, -5)))
+        .expect("insert should work");
+    tree.insert(QuadTreeElement::new(1001, AABB::new(-20, -20, -18, -18)))
+        .expect("insert should work");
     // top-right
-    tree.insert(QuadTreeElement::new(2000, AABB::new(5, -15, 15, -5)));
+    tree.insert(QuadTreeElement::new(2000, AABB::new(5, -15, 15, -5)))
+        .expect("insert should work");
     // bottom-left
-    tree.insert(QuadTreeElement::new(3000, AABB::new(-15, 5, -5, 15)));
+    tree.insert(QuadTreeElement::new(3000, AABB::new(-15, 5, -5, 15)))
+        .expect("insert should work");
     // bottom-right
-    tree.insert(QuadTreeElement::new(4000, AABB::new(5, 5, 15, 15)));
+    tree.insert(QuadTreeElement::new(4000, AABB::new(5, 5, 15, 15)))
+        .expect("insert should work");
     // center
-    tree.insert(QuadTreeElement::new(5000, AABB::new(-5, -5, 5, 5)));
+    tree.insert(QuadTreeElement::new(5000, AABB::new(-5, -5, 5, 5)))
+        .expect("insert should work");
 
     // The depth of 1 limits the tree to four quadrants.
     // Each of the first five elements creates a single reference
@@ -684,18 +681,24 @@ mod test {
     #[test]
     fn cleanup_works() {
         let quad_rect = QuadRect::new(-20, -20, 40, 40);
-        let mut tree = QuadTree::new(quad_rect, 1);
+        let mut tree = QuadTree::new(quad_rect, 1, 1, 1);
         // top-left
-        tree.insert(QuadTreeElement::new(1000, AABB::new(-15, -15, -5, -5)));
-        tree.insert(QuadTreeElement::new(1001, AABB::new(-20, -20, -18, -18)));
+        tree.insert(QuadTreeElement::new(1000, AABB::new(-15, -15, -5, -5)))
+            .expect("insert should work");
+        tree.insert(QuadTreeElement::new(1001, AABB::new(-20, -20, -18, -18)))
+            .expect("insert should work");
         // top-right
-        tree.insert(QuadTreeElement::new(2000, AABB::new(5, -15, 15, -5)));
+        tree.insert(QuadTreeElement::new(2000, AABB::new(5, -15, 15, -5)))
+            .expect("insert should work");
         // bottom-left
-        tree.insert(QuadTreeElement::new(3000, AABB::new(-15, 5, -5, 15)));
+        tree.insert(QuadTreeElement::new(3000, AABB::new(-15, 5, -5, 15)))
+            .expect("insert should work");
         // bottom-right
-        tree.insert(QuadTreeElement::new(4000, AABB::new(5, 5, 15, 15)));
+        tree.insert(QuadTreeElement::new(4000, AABB::new(5, 5, 15, 15)))
+            .expect("insert should work");
         // center
-        tree.insert(QuadTreeElement::new(5000, AABB::new(-5, -5, 5, 5)));
+        tree.insert(QuadTreeElement::new(5000, AABB::new(-5, -5, 5, 5)))
+            .expect("insert should work");
 
         // Similar to index test.
         assert_eq!(tree.collect_ids().len(), 6);
